@@ -5,7 +5,7 @@ use warnings;
 
 use vars qw($VERSION);
 
-$VERSION = '2.16';
+$VERSION = '2.22';
 
 use POE;
 use POE::Wheel::SocketFactory;
@@ -36,7 +36,6 @@ BEGIN {
 }
 
 use MooseX::POE;
-use MooseX::AttributeHelpers;
 use Moose::Util::TypeConstraints;
 
 
@@ -48,7 +47,7 @@ has 'address' => (
   is => 'ro',
 );
 
-has 'port' => ( 
+has 'port' => (
   is => 'ro',
   default => sub { 0 },
   writer => '_set_port',
@@ -119,14 +118,14 @@ has 'setuphandler' => (
 );
 
 has 'retries' => (
-  metaclass => 'Counter',
+  traits  => ['Counter'],
   is        => 'ro',
   isa       => 'Num',
   default   => sub { 0 },
-  provides  => {
-    inc => 'inc_retry',
-    dec => 'dec_retry',          
-    reset => 'reset_retries',
+  handles   => {
+    inc_retry     => 'inc',
+    dec_retry     => 'dec',
+    reset_retries => 'reset',
   },
 );
 
@@ -176,7 +175,7 @@ sub BUILDARGS {
   $args{lc $_} = delete $args{$_} for keys %args;
 
   if ( $args{sslkeycert} and ref $args{sslkeycert} eq 'ARRAY'
-	and scalar @{ $args{sslkeycert} } == 2 ) {  
+	and scalar @{ $args{sslkeycert} } == 2 ) {
 
      eval {
        require POE::Component::SSLify;
@@ -253,8 +252,10 @@ event 'SHUTDOWN' => sub {
    # Are we gracefully shutting down or not?
    if ( $graceful ) {
 
-      # Check for number of requests
-      if ( keys( %{ $self->_requests } ) == 0 ) {
+      # Check for existing requests and keep-alive connections
+      if ( keys( %{ $self->_requests } ) == 0
+         and keys( %{ $self->_connections } ) == 0 )
+      {
 
          # Alright, shutdown anyway
 
@@ -797,11 +798,11 @@ sub must_keepalive {
    return 0 if $resp->is_error;
 
    # Connection is a comma-seperated header
-   my $conn = lc $req->header('Connection');
+   my $conn = lc ($req->header('Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
-   $conn = lc $req->header('Proxy-Connection');
+   $conn = lc ($req->header('Proxy-Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
-   $conn = lc $resp->header('Connection');
+   $conn = lc ($resp->header('Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
 
    # HTTP/1.1 = keep
@@ -1451,7 +1452,7 @@ ARG1 -> the IP address of the client
 
 =item C<LOG2HANDLER>
 
-Expect a hashref with the following key, valyes:
+Expect a hashref with the following key, values:
 
 SESSION	->	The session to send the input
 
@@ -1616,7 +1617,7 @@ $response to a streaming state and once you trigger it:
    # send a CLOSE event to the kernel with the appropriate response as parameter
    $kernel->yield('GOT_STREAM', $response);
 
-The optionnal dont_flush option gives the user the ability to control the callback
+The optional dont_flush option gives the user the ability to control the callback
 to the streaming event, which means once your stream event has reached its end
 it won't be called, you have to call it back.
 
@@ -1638,7 +1639,7 @@ shutdown when your streaming is done (EOF for example).
       #
       # $kernel->delay('GOT_STREAM', 1, $stream );
 
-      # otherwise the GOT_STREAM event is triggered continously until
+      # otherwise the GOT_STREAM event is triggered continuously until
       # we call the CLOSE event on the response like that :
       #
       if ($heap{'streaming_is_done'}) {
